var bcrypt = require("bcrypt");
var mongoose = require('mongoose');
var User;
var db = mongoose.connection;

mongoose.connect('mongodb://localhost/users');
db.on('error', console.error.bind(console, 'connection error:'));

var setupUserDB = function () {
    var userSchema = mongoose.Schema({
	username: String,
	password: String
    });
    User = mongoose.model("User", userSchema);
}
db.once('open', setupUserDB);

var index = function(req, res){
    res.render('index', { title: 'COMP 2406 Authenticated Sessions Demo',
			  error: req.query.error });
}

var redirectLoggedIn = function(req, res, next) {
    if (req.session.username) {
        res.redirect("/users");
    } else {
        next();
    }
}

var redirectNotLoggedIn = function(req, res, next) {
    if (req.session.username) {
        next();
    } else {
        res.redirect("/");
    }
}

var register = function(req, res) {
    var username = req.body.username;
    var password = req.body.password;

    var addUser = function(err, users) {
	if(users.length!=0){
	    res.redirect("/?error=user already exists");	
	    return;
	}
	
	//generate a salt, with 10 rounds (2^10 iterations)
	bcrypt.genSalt(10, function(err, salt) {
	    bcrypt.hash(password, salt, function(err, hash) {
      		var newUser = new User({
      		    username: username,
      		    password: hash
      		});	
      		
      		newUser.save(function(err, newUser){
      		    res.render('registered', { username: newUser.username });
      		});    
	    });
	});	
    };	
    
    User.find({username: username}, addUser);
}

var login = function(req, res){
    var username = req.body.username;
    var password = req.body.password;
    //Search the Database for a User with the given username
    User.find({username: username}, function(err, users){
	//we couldn't find a user with that name
	if(err || users.length==0){
	    res.redirect("/?error=invalid username or password");	
	    return;
	}
	
	var user = users[0];
	//compare the hash we have for the user with what this password hashes to
	bcrypt.compare(password, user.password, function(err, authenticated){
	    if(authenticated){
		req.session.username = user.username;
		res.redirect("/users");
	    }else{
		res.redirect("/?error=invalid username or password");	
	    }
	});
    });
}

var logout = function(req, res){
    req.session.destroy(function(err){
	if(err){
            console.log("Error: %s", err);
	}
	res.redirect("/");
    });	
}

exports.index = index;
exports.redirectLoggedIn = redirectLoggedIn;
exports.redirectNotLoggedIn = redirectNotLoggedIn;
exports.register = register;
exports.login = login;
exports.logout = logout;
